@@ -27,6 +27,8 @@
 #include "system/network.h"
 #include "system/locale.h"
 #include "system/filesys.h"
+#include "system/select.h"
+#include "lib/util/select.h"
 #include "lib/util/util_net.h"
 
 #undef strcasecmp
@@ -365,7 +367,7 @@ uint32_t interpret_addr(const char *str)
 		ret = dest.s_addr; /* NETWORK BYTE ORDER ! */
 	} else {
 		/* Otherwise assume it's a network name of some sort and use
-			getadddrinfo. */
+			getaddrinfo. */
 		struct addrinfo *res = NULL;
 		struct addrinfo *res_list = NULL;
 		if (!interpret_string_addr_internal(&res_list,
@@ -640,6 +642,9 @@ bool is_zero_addr(const struct sockaddr_storage *pss)
 		const struct in_addr *pin = &((const struct sockaddr_in *)pss)->sin_addr;
 		return is_zero_ip_v4(*pin);
 	}
+	if (pss->ss_family == AF_UNSPEC) {
+		return true;
+	}
 	return false;
 }
 
@@ -977,28 +982,43 @@ static const smb_socket_option socket_options[] = {
  Print socket options.
 ****************************************************************************/
 
-static void print_socket_options(int s)
+static void print_socket_options(TALLOC_CTX *ctx, int s)
 {
-	int value;
-	socklen_t vlen = 4;
 	const smb_socket_option *p = &socket_options[0];
+	char *str = NULL;
 
-	/* wrapped in if statement to prevent streams
-	 * leak in SCO Openserver 5.0 */
-	/* reported on samba-technical  --jerry */
-	if ( DEBUGLEVEL >= 5 ) {
-		DEBUG(5,("Socket options:\n"));
-		for (; p->name != NULL; p++) {
-			if (getsockopt(s, p->level, p->option,
-						(void *)&value, &vlen) == -1) {
-				DEBUGADD(5,("\tCould not test socket option %s.\n",
-							p->name));
-			} else {
-				DEBUGADD(5,("\t%s = %d\n",
-							p->name,value));
-			}
-		}
+	if (DEBUGLEVEL < 5) {
+		return;
 	}
+
+	str = talloc_strdup(ctx, "");
+	if (str == NULL) {
+		DBG_WARNING("talloc failed\n");
+		goto done;
+	}
+
+	for (; p->name != NULL; p++) {
+		int ret, val;
+		socklen_t vlen = sizeof(val);
+
+		ret = getsockopt(s, p->level, p->option, (void *)&val, &vlen);
+		if (ret == -1) {
+			DBG_INFO("Could not test socket option %s: %s.\n",
+				 p->name, strerror(errno));
+			continue;
+		}
+
+		talloc_asprintf_addbuf(
+			&str,
+			"%s%s=%d",
+			str[0] != '\0' ? ", " : "",
+			p->name,
+			val);
+	}
+
+	DEBUG(5, ("socket options: %s\n", str));
+done:
+	TALLOC_FREE(str);
  }
 
 /****************************************************************************
@@ -1062,8 +1082,8 @@ void set_socket_options(int fd, const char *options)
 		}
 	}
 
+	print_socket_options(ctx, fd);
 	TALLOC_FREE(ctx);
-	print_socket_options(fd);
 }
 
 /*
@@ -1095,12 +1115,12 @@ bool sockaddr_storage_to_samba_sockaddr(
 bool samba_sockaddr_set_port(struct samba_sockaddr *sa, uint16_t port)
 {
 	if (sa->u.sa.sa_family == AF_INET) {
-		sa->u.in.sin_port = port;
+		sa->u.in.sin_port = htons(port);
 		return true;
 	}
 #ifdef HAVE_IPV6
 	if (sa->u.sa.sa_family == AF_INET6) {
-		sa->u.in6.sin6_port = port;
+		sa->u.in6.sin6_port = htons(port);
 		return true;
 	}
 #endif
@@ -1110,14 +1130,111 @@ bool samba_sockaddr_set_port(struct samba_sockaddr *sa, uint16_t port)
 bool samba_sockaddr_get_port(const struct samba_sockaddr *sa, uint16_t *port)
 {
 	if (sa->u.sa.sa_family == AF_INET) {
-		*port = sa->u.in.sin_port;
+		*port = ntohs(sa->u.in.sin_port);
 		return true;
 	}
 #ifdef HAVE_IPV6
 	if (sa->u.sa.sa_family == AF_INET6) {
-		*port = sa->u.in6.sin6_port;
+		*port = ntohs(sa->u.in6.sin6_port);
 		return true;
 	}
 #endif
 	return false;
 }
+
+int samba_socket_poll_error(int fd)
+{
+	struct pollfd pfd = {
+		.fd = fd,
+#ifdef POLLRDHUP
+		.events = POLLRDHUP, /* POLLERR and POLLHUP are not needed */
+#endif
+	};
+	int ret;
+
+	errno = 0;
+	ret = sys_poll_intr(&pfd, 1, 0);
+	if (ret == 0) {
+		return 0;
+	}
+	if (ret != 1) {
+		return POLLNVAL;
+	}
+
+	if (pfd.revents & POLLERR) {
+		return POLLERR;
+	}
+	if (pfd.revents & POLLHUP) {
+		return POLLHUP;
+	}
+#ifdef POLLRDHUP
+	if (pfd.revents & POLLRDHUP) {
+		return POLLRDHUP;
+	}
+#endif
+
+	/* should never be reached! */
+	return POLLNVAL;
+}
+
+int samba_socket_sock_error(int fd)
+{
+	int ret, error = 0;
+	socklen_t len = sizeof(error);
+
+	/*
+	 * if no data is available check if the socket is in error state. For
+	 * dgram sockets it's the way to return ICMP error messages of
+	 * connected sockets to the caller.
+	 */
+	ret = getsockopt(fd, SOL_SOCKET, SO_ERROR, &error, &len);
+	if (ret == -1) {
+		return ret;
+	}
+	if (error != 0) {
+		errno = error;
+		return -1;
+	}
+	return 0;
+}
+
+int samba_socket_poll_or_sock_error(int fd)
+{
+	int ret;
+	int poll_error = 0;
+
+	poll_error = samba_socket_poll_error(fd);
+	if (poll_error == 0) {
+		return 0;
+	}
+
+#ifdef POLLRDHUP
+	if (poll_error == POLLRDHUP) {
+		errno = ECONNRESET;
+		return -1;
+	}
+#endif
+
+	if (poll_error == POLLHUP) {
+		errno = EPIPE;
+		return -1;
+	}
+
+	/*
+	 * POLLERR and POLLNVAL fallback to
+	 * getsockopt(fd, SOL_SOCKET, SO_ERROR)
+	 * and force EPIPE as fallback.
+	 */
+
+	errno = 0;
+	ret = samba_socket_sock_error(fd);
+	if (ret == 0) {
+		errno = EPIPE;
+	}
+
+	if (errno == 0) {
+		errno = EPIPE;
+	}
+
+	return -1;
+}
