@@ -25,6 +25,7 @@
 #include "system/select.h"
 #include "libsmb/libsmb.h"
 #include "libsmb/unexpected.h"
+#include "lib/util/string_wrappers.h"
 
 extern int ClientNMB;
 extern int ClientDGRAM;
@@ -42,6 +43,7 @@ bool nmbd_init_packet_server(void)
 
 	status = nb_packet_server_create(
 		NULL, nmbd_event_context(),
+		global_nmbd_socket_dir(),
 		lp_parm_int(-1, "nmbd", "unexpected_clients", 200),
 		&packet_server);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -1879,6 +1881,49 @@ static void nmbd_fd_handler(struct tevent_context *ev,
 	attr->triggered = true;
 }
 
+/****************************************************************************
+ Read from a socket.
+****************************************************************************/
+
+static ssize_t read_udp_v4_socket(
+	int fd,
+	char *buf,
+	size_t len,
+	struct sockaddr_storage *psa)
+{
+	ssize_t ret;
+	socklen_t socklen = sizeof(*psa);
+	struct sockaddr_in *si = (struct sockaddr_in *)psa;
+
+	memset((char *)psa,'\0',socklen);
+
+	ret = (ssize_t)sys_recvfrom(fd,buf,len,0,
+			(struct sockaddr *)psa,&socklen);
+	if (ret <= 0) {
+		/* Don't print a low debug error for a non-blocking socket. */
+		if (errno == EAGAIN) {
+			DEBUG(10,("read_udp_v4_socket: returned EAGAIN\n"));
+		} else {
+			DEBUG(2,("read_udp_v4_socket: failed. errno=%s\n",
+				strerror(errno)));
+		}
+		return 0;
+	}
+
+	if (psa->ss_family != AF_INET) {
+		DEBUG(2,("read_udp_v4_socket: invalid address family %d "
+			"(not IPv4)\n", (int)psa->ss_family));
+		return 0;
+	}
+
+	DEBUG(10,("read_udp_v4_socket: ip %s port %d read: %lu\n",
+			inet_ntoa(si->sin_addr),
+			si->sin_port,
+			(unsigned long)ret));
+
+	return ret;
+}
+
 /*******************************************************************
  Read a packet from a socket and parse it, returning a packet ready
  to be used or put on the queue. This assumes a UDP socket.
