@@ -29,8 +29,8 @@
 #include "fake_file.h"
 #include "smbd/smbd.h"
 #include "smbd/globals.h"
-#include "lib/util/memcache.h"
-#include "libcli/smb/reparse_symlink.h"
+#include "libcli/smb/reparse.h"
+#include "source3/smbd/dir.h"
 
 uint32_t ucf_flags_from_smb_request(struct smb_request *req)
 {
@@ -41,11 +41,7 @@ uint32_t ucf_flags_from_smb_request(struct smb_request *req)
 	}
 
 	if (req->posix_pathnames) {
-		ucf_flags |= UCF_POSIX_PATHNAMES;
-
-		if (!req->sconn->using_smb2) {
-			ucf_flags |= UCF_LCOMP_LNK_OK;
-		}
+		ucf_flags |= (UCF_POSIX_PATHNAMES|UCF_LCOMP_LNK_OK);
 	}
 	if (req->flags2 & FLAGS2_DFS_PATHNAMES) {
 		ucf_flags |= UCF_DFS_PATHNAME;
@@ -57,7 +53,9 @@ uint32_t ucf_flags_from_smb_request(struct smb_request *req)
 	return ucf_flags;
 }
 
-uint32_t filename_create_ucf_flags(struct smb_request *req, uint32_t create_disposition)
+uint32_t filename_create_ucf_flags(struct smb_request *req,
+				   uint32_t create_disposition,
+				   uint32_t create_options)
 {
 	uint32_t ucf_flags = 0;
 
@@ -75,6 +73,10 @@ uint32_t filename_create_ucf_flags(struct smb_request *req, uint32_t create_disp
 		break;
 	}
 
+	if (create_options & FILE_OPEN_REPARSE_POINT) {
+		ucf_flags |= UCF_LCOMP_LNK_OK;
+	}
+
 	return ucf_flags;
 }
 
@@ -326,7 +328,6 @@ NTSTATUS get_real_filename_full_scan_at(struct files_struct *dirfsp,
 	/* now scan for matching names */
 	while ((dname = ReadDirName(cur_dir, &talloced))) {
 
-		/* Is it dot or dot dot. */
 		if (ISDOT(dname) || ISDOTDOT(dname)) {
 			TALLOC_FREE(talloced);
 			continue;
@@ -405,53 +406,6 @@ NTSTATUS get_real_filename_at(struct files_struct *dirfsp,
 	return status;
 }
 
-/*
- * Create the memcache-key for GETREALFILENAME_CACHE: This supplements
- * the stat cache for the last component to be looked up. Cache
- * contents is the correctly capitalized translation of the parameter
- * "name" as it exists on disk. This is indexed by inode of the dirfsp
- * and name, and contrary to stat_cahce_lookup() it does not
- * vfs_stat() the last component. This will be taken care of by an
- * attempt to do a openat_pathref_fsp().
- */
-static bool get_real_filename_cache_key(
-	TALLOC_CTX *mem_ctx,
-	struct files_struct *dirfsp,
-	const char *name,
-	DATA_BLOB *_key)
-{
-	struct file_id fid = vfs_file_id_from_sbuf(
-		dirfsp->conn, &dirfsp->fsp_name->st);
-	char *upper = NULL;
-	uint8_t *key = NULL;
-	size_t namelen, keylen;
-
-	upper = talloc_strdup_upper(mem_ctx, name);
-	if (upper == NULL) {
-		return false;
-	}
-	namelen = talloc_get_size(upper);
-
-	keylen = namelen + sizeof(fid);
-	if (keylen < sizeof(fid)) {
-		TALLOC_FREE(upper);
-		return false;
-	}
-
-	key = talloc_size(mem_ctx, keylen);
-	if (key == NULL) {
-		TALLOC_FREE(upper);
-		return false;
-	}
-
-	memcpy(key, &fid, sizeof(fid));
-	memcpy(key + sizeof(fid), upper, namelen);
-	TALLOC_FREE(upper);
-
-	*_key = (DATA_BLOB) { .data = key, .length = keylen, };
-	return true;
-}
-
 /*
  * Lightweight function to just get last component
  * for rename / enumerate directory calls.
@@ -483,93 +437,6 @@ char *get_original_lcomp(TALLOC_CTX *ctx,
 	return orig_lcomp;
 }
 
-/*
- * Deal with the SMB1 semantics of sending a pathname with a
- * wildcard as the terminal component for a SMB1search or
- * trans2 findfirst.
- */
-
-NTSTATUS filename_convert_smb1_search_path(TALLOC_CTX *ctx,
-					   connection_struct *conn,
-					   char *name_in,
-					   uint32_t ucf_flags,
-					   struct files_struct **_dirfsp,
-					   struct smb_filename **_smb_fname_out,
-					   char **_mask_out)
-{
-	NTSTATUS status;
-	char *p = NULL;
-	char *mask = NULL;
-	struct smb_filename *smb_fname = NULL;
-	NTTIME twrp = 0;
-
-	*_smb_fname_out = NULL;
-	*_dirfsp = NULL;
-	*_mask_out = NULL;
-
-	DBG_DEBUG("name_in: %s\n", name_in);
-
-	if (ucf_flags & UCF_GMT_PATHNAME) {
-		extract_snapshot_token(name_in, &twrp);
-		ucf_flags &= ~UCF_GMT_PATHNAME;
-	}
-
-	/* Get the original lcomp. */
-	mask = get_original_lcomp(ctx,
-				  conn,
-				  name_in,
-				  ucf_flags);
-	if (mask == NULL) {
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	if (mask[0] == '\0') {
-		/* Windows and OS/2 systems treat search on the root as * */
-		TALLOC_FREE(mask);
-		mask = talloc_strdup(ctx, "*");
-		if (mask == NULL) {
-			return NT_STATUS_NO_MEMORY;
-		}
-	}
-
-	DBG_DEBUG("mask = %s\n", mask);
-
-	/*
-	 * Remove the terminal component so
-	 * filename_convert_dirfsp never sees the mask.
-	 */
-	p = strrchr_m(name_in,'/');
-	if (p == NULL) {
-		/* filename_convert_dirfsp handles a '\0' name. */
-		name_in[0] = '\0';
-	} else {
-		*p = '\0';
-	}
-
-	DBG_DEBUG("For filename_convert_dirfsp: name_in = %s\n",
-		name_in);
-
-	/* Convert the parent directory path. */
-	status = filename_convert_dirfsp(ctx,
-					 conn,
-					 name_in,
-					 ucf_flags,
-					 twrp,
-					 _dirfsp,
-					 &smb_fname);
-
-	if (!NT_STATUS_IS_OK(status)) {
-		DBG_DEBUG("filename_convert error for %s: %s\n",
-			name_in,
-			nt_errstr(status));
-	}
-
-	*_smb_fname_out = talloc_move(ctx, &smb_fname);
-	*_mask_out = talloc_move(ctx, &mask);
-
-	return status;
-}
-
 /*
  * Get the correct capitalized stream name hanging off
  * base_fsp. Equivalent of get_real_filename(), but for streams.
@@ -728,220 +595,6 @@ static NTSTATUS filename_convert_normalize_new(
 	return NT_STATUS_OK;
 }
 
-/*
- * Open smb_fname_rel->fsp as a pathref fsp with a case insensitive
- * fallback using GETREALFILENAME_CACHE and get_real_filename_at() if
- * the first attempt based on the filename sent by the client gives
- * ENOENT.
- */
-static NTSTATUS openat_pathref_fsp_case_insensitive(
-	struct files_struct *dirfsp,
-	struct smb_filename *smb_fname_rel,
-	uint32_t ucf_flags)
-{
-	const bool posix = (ucf_flags & UCF_POSIX_PATHNAMES);
-	DATA_BLOB cache_key = { .data = NULL, };
-	char *found_name = NULL;
-	NTSTATUS status;
-	bool ok;
-
-	SET_STAT_INVALID(smb_fname_rel->st);
-
-	/* Check veto files - only looks at last component. */
-	if (IS_VETO_PATH(dirfsp->conn, smb_fname_rel->base_name)) {
-		DBG_DEBUG("veto files rejecting last component %s\n",
-			  smb_fname_str_dbg(smb_fname_rel));
-		return NT_STATUS_NETWORK_OPEN_RESTRICTION;
-	}
-
-	status = openat_pathref_fsp(dirfsp, smb_fname_rel);
-
-	if (NT_STATUS_IS_OK(status)) {
-		return NT_STATUS_OK;
-	}
-
-	if (VALID_STAT(smb_fname_rel->st)) {
-		/*
-		 * We got an error although the object existed. Might
-		 * be a symlink we don't want.
-		 */
-		return status;
-	}
-
-	if (!NT_STATUS_EQUAL(status, NT_STATUS_OBJECT_NAME_NOT_FOUND)) {
-		/*
-		 * Only retry on ENOENT
-		 */
-		return status;
-	}
-
-	if (posix || dirfsp->conn->case_sensitive) {
-		/*
-		 * Only return case insensitive if required
-		 */
-		return status;
-	}
-
-	if (lp_stat_cache()) {
-		char *base_name = smb_fname_rel->base_name;
-		char *original_relname = NULL;
-		DATA_BLOB value = { .data = NULL };
-
-		ok = get_real_filename_cache_key(
-			talloc_tos(), dirfsp, base_name, &cache_key);
-		if (!ok) {
-			/*
-			 * probably ENOMEM, just bail
-			 */
-			return status;
-		}
-
-		DO_PROFILE_INC(statcache_lookups);
-
-		ok = memcache_lookup(
-			NULL, GETREALFILENAME_CACHE, cache_key, &value);
-		if (!ok) {
-			DO_PROFILE_INC(statcache_misses);
-			goto lookup;
-		}
-		DO_PROFILE_INC(statcache_hits);
-
-		/*
-		 * For the "new filename" case we need to preserve the
-		 * capitalization the client sent us, see
-		 * https://bugzilla.samba.org/show_bug.cgi?id=15481
-		 */
-		original_relname = smb_fname_rel->base_name;
-
-		smb_fname_rel->base_name = talloc_memdup(
-			smb_fname_rel, value.data, value.length);
-		if (smb_fname_rel->base_name == NULL) {
-			TALLOC_FREE(cache_key.data);
-			return NT_STATUS_NO_MEMORY;
-		}
-
-		if (IS_VETO_PATH(dirfsp->conn, smb_fname_rel->base_name)) {
-			DBG_DEBUG("veto files rejecting last component %s\n",
-				  smb_fname_str_dbg(smb_fname_rel));
-			TALLOC_FREE(cache_key.data);
-			return NT_STATUS_NETWORK_OPEN_RESTRICTION;
-		}
-
-		status = openat_pathref_fsp(dirfsp, smb_fname_rel);
-		if (NT_STATUS_IS_OK(status)) {
-			TALLOC_FREE(cache_key.data);
-			TALLOC_FREE(original_relname);
-			return NT_STATUS_OK;
-		}
-
-		memcache_delete(NULL, GETREALFILENAME_CACHE, cache_key);
-		TALLOC_FREE(smb_fname_rel->base_name);
-		smb_fname_rel->base_name = original_relname;
-	}
-
-lookup:
-	status = get_real_filename_at(
-		dirfsp, smb_fname_rel->base_name, smb_fname_rel, &found_name);
-	if (NT_STATUS_EQUAL(status, NT_STATUS_ACCESS_DENIED) &&
-	    (ucf_flags & UCF_PREP_CREATEFILE)) {
-		/*
-		 * dropbox
-		 */
-		status = NT_STATUS_OBJECT_NAME_NOT_FOUND;
-	}
-
-	if (NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(smb_fname_rel->base_name);
-		smb_fname_rel->base_name = found_name;
-
-		if (IS_VETO_PATH(dirfsp->conn, smb_fname_rel->base_name)) {
-			DBG_DEBUG("veto files rejecting last component %s\n",
-				smb_fname_str_dbg(smb_fname_rel));
-			return NT_STATUS_NETWORK_OPEN_RESTRICTION;
-		}
-
-		status = openat_pathref_fsp(dirfsp, smb_fname_rel);
-	}
-
-	if (NT_STATUS_IS_OK(status) && (cache_key.data != NULL)) {
-		DATA_BLOB value = {
-			.data = (uint8_t *)smb_fname_rel->base_name,
-			.length = strlen(smb_fname_rel->base_name) + 1,
-		};
-
-		memcache_add(NULL, GETREALFILENAME_CACHE, cache_key, value);
-	}
-
-	TALLOC_FREE(cache_key.data);
-
-	return status;
-}
-
-static const char *previous_slash(const char *name_in, const char *slash)
-{
-	const char *prev = name_in;
-
-	while (true) {
-		const char *next = strchr_m(prev, '/');
-
-		SMB_ASSERT(next != NULL); /* we have at least one slash */
-
-		if (next == slash) {
-			break;
-		}
-
-		prev = next+1;
-	};
-
-	if (prev == name_in) {
-		/* no previous slash */
-		return NULL;
-	}
-
-	return prev;
-}
-
-static char *symlink_target_path(
-	TALLOC_CTX *mem_ctx,
-	const char *name_in,
-	const char *substitute,
-	size_t unparsed)
-{
-	size_t name_in_len = strlen(name_in);
-	const char *p_unparsed = NULL;
-	const char *parent = NULL;
-	char *ret;
-
-	SMB_ASSERT(unparsed <= name_in_len);
-
-	p_unparsed = name_in + (name_in_len - unparsed);
-
-	if (substitute[0] == '/') {
-		ret = talloc_asprintf(mem_ctx, "%s%s", substitute, p_unparsed);
-		return ret;
-	}
-
-	if (unparsed == 0) {
-		parent = strrchr_m(name_in, '/');
-	} else {
-		parent = previous_slash(name_in, p_unparsed);
-	}
-
-	if (parent == NULL) {
-		/* no previous slash */
-		parent = name_in;
-	}
-
-	ret = talloc_asprintf(
-		mem_ctx,
-		"%.*s%s%s",
-		(int)(parent - name_in),
-		name_in,
-		substitute,
-		p_unparsed);
-	return ret;
-}
-
 NTSTATUS safe_symlink_target_path(TALLOC_CTX *mem_ctx,
 				  const char *connectpath,
 				  const char *dir,
@@ -992,7 +645,8 @@ NTSTATUS safe_symlink_target_path(TALLOC_CTX *mem_ctx,
 		connectpath, strlen(connectpath), abs_target_canon, &relative);
 	if (!in_share) {
 		DBG_DEBUG("wide link to %s\n", abs_target_canon);
-		status = NT_STATUS_OBJECT_PATH_NOT_FOUND;
+		status = (unparsed != 0) ? NT_STATUS_OBJECT_PATH_NOT_FOUND
+					 : NT_STATUS_OBJECT_NAME_NOT_FOUND;
 		goto fail;
 	}
 
@@ -1011,20 +665,22 @@ fail:
  * Split up name_in as sent by the client into a directory pathref fsp
  * and a relative smb_filename.
  */
-static NTSTATUS filename_convert_dirfsp_nosymlink(
-	TALLOC_CTX *mem_ctx,
-	connection_struct *conn,
-	const char *name_in,
-	uint32_t ucf_flags,
-	NTTIME twrp,
-	struct files_struct **_dirfsp,
-	struct smb_filename **_smb_fname,
-	char **_substitute,
-	size_t *_unparsed)
+NTSTATUS
+filename_convert_dirfsp_nosymlink(TALLOC_CTX *mem_ctx,
+				  connection_struct *conn,
+				  struct files_struct *basedir,
+				  const char *name_in,
+				  uint32_t ucf_flags,
+				  NTTIME twrp,
+				  struct files_struct **_dirfsp,
+				  struct smb_filename **_smb_fname,
+				  struct smb_filename **_smb_fname_rel,
+				  struct reparse_data_buffer **_symlink_err)
 {
 	struct smb_filename *smb_dirname = NULL;
 	struct smb_filename *smb_fname_rel = NULL;
 	struct smb_filename *smb_fname = NULL;
+	struct reparse_data_buffer *symlink_err = NULL;
 	const bool posix = (ucf_flags & UCF_POSIX_PATHNAMES);
 	char *dirname = NULL;
 	const char *fname_rel = NULL;
@@ -1037,21 +693,21 @@ static NTSTATUS filename_convert_dirfsp_nosymlink(
 	SMB_ASSERT(!(ucf_flags & UCF_DFS_PATHNAME));
 
 	if (is_fake_file_path(name_in)) {
+		const struct timespec omit = make_omit_timespec();
 		smb_fname = synthetic_smb_fname_split(mem_ctx, name_in, posix);
 		if (smb_fname == NULL) {
 			return NT_STATUS_NO_MEMORY;
 		}
-		smb_fname->st = (SMB_STRUCT_STAT) { .st_ex_nlink = 1 };
-		smb_fname->st.st_ex_btime =
-			(struct timespec){0, SAMBA_UTIME_OMIT};
-		smb_fname->st.st_ex_atime =
-			(struct timespec){0, SAMBA_UTIME_OMIT};
-		smb_fname->st.st_ex_mtime =
-			(struct timespec){0, SAMBA_UTIME_OMIT};
-		smb_fname->st.st_ex_ctime =
-			(struct timespec){0, SAMBA_UTIME_OMIT};
+		smb_fname->st = (SMB_STRUCT_STAT){
+			.st_ex_nlink = 1,
+			.st_ex_mode = S_IFREG | 0644,
+			.st_ex_btime = omit,
+			.st_ex_atime = omit,
+			.st_ex_mtime = omit,
+			.st_ex_ctime = omit,
+		};
 
-		*_dirfsp = conn->cwd_fsp;
+		*_dirfsp = basedir;
 		*_smb_fname = smb_fname;
 		return NT_STATUS_OK;
 	}
@@ -1097,18 +753,10 @@ static NTSTATUS filename_convert_dirfsp_nosymlink(
 	}
 
 	if (dirname[0] == '\0') {
-		status = synthetic_pathref(
-			mem_ctx,
-			conn->cwd_fsp,
-			".",
-			NULL,
-			NULL,
-			0,
-			posix ? SMB_FILENAME_POSIX_PATH : 0,
-			&smb_dirname);
+		status = openat_pathref_fsp_dot(
+			mem_ctx, basedir,
+			posix ? SMB_FILENAME_POSIX_PATH : 0, &smb_dirname);
 	} else {
-		struct open_symlink_err *symlink_err = NULL;
-
 		status = normalize_filename_case(conn, dirname, ucf_flags);
 		if (!NT_STATUS_IS_OK(status)) {
 			DBG_ERR("normalize_filename_case %s failed: %s\n",
@@ -1119,7 +767,7 @@ static NTSTATUS filename_convert_dirfsp_nosymlink(
 
 		status = openat_pathref_fsp_nosymlink(mem_ctx,
 						      conn,
-						      conn->cwd_fsp,
+						      basedir,
 						      dirname,
 						      twrp,
 						      posix,
@@ -1127,27 +775,24 @@ static NTSTATUS filename_convert_dirfsp_nosymlink(
 						      &symlink_err);
 
 		if (NT_STATUS_EQUAL(status, NT_STATUS_STOPPED_ON_SYMLINK)) {
+			struct symlink_reparse_struct
+				*lnk = &symlink_err->parsed.lnk;
+			size_t unparsed = lnk->unparsed_path_length;
 			size_t name_in_len, dirname_len;
 
-			if (lp_host_msdfs() && lp_msdfs_root(SNUM(conn)) &&
-			    strnequal(symlink_err->reparse->substitute_name,
-				      "msdfs:",
-				      6)) {
-				status = NT_STATUS_PATH_NOT_COVERED;
-				goto fail;
-			}
-
 			name_in_len = strlen(name_in);
 			dirname_len = strlen(dirname);
 
 			SMB_ASSERT(name_in_len >= dirname_len);
 
-			*_substitute = talloc_move(
-				mem_ctx,
-				&symlink_err->reparse->substitute_name);
-			*_unparsed = symlink_err->unparsed +
-				     (name_in_len - dirname_len);
+			unparsed += (name_in_len - dirname_len);
 
+			if (unparsed > UINT16_MAX) {
+				status = NT_STATUS_BUFFER_OVERFLOW;
+				goto fail;
+			}
+
+			lnk->unparsed_path_length = unparsed;
 			goto fail;
 		}
 	}
@@ -1232,45 +877,27 @@ static NTSTATUS filename_convert_dirfsp_nosymlink(
 		goto fail;
 	}
 
-	status = openat_pathref_fsp_case_insensitive(
-		smb_dirname->fsp, smb_fname_rel, ucf_flags);
+	status = openat_pathref_fsp_lcomp(smb_dirname->fsp,
+					  smb_fname_rel,
+					  ucf_flags);
 
-	if (NT_STATUS_EQUAL(status, NT_STATUS_OBJECT_NAME_NOT_FOUND) &&
-	    VALID_STAT(smb_fname_rel->st) &&
-	    S_ISLNK(smb_fname_rel->st.st_ex_mode)) {
+	if (NT_STATUS_IS_OK(status) && S_ISLNK(smb_fname_rel->st.st_ex_mode)) {
 
 		/*
-		 * If we're on an MSDFS share, see if this is
-		 * an MSDFS link.
+		 * Upper layers might need the link target. Here we
+		 * still have the relname around, get the symlink err.
 		 */
-		if (lp_host_msdfs() &&
-		    lp_msdfs_root(SNUM(conn)) &&
-		    is_msdfs_link(smb_dirname->fsp, smb_fname_rel))
-		{
-			status = NT_STATUS_PATH_NOT_COVERED;
+		status = read_symlink_reparse(mem_ctx,
+					      smb_dirname->fsp,
+					      smb_fname_rel,
+					      &symlink_err);
+		if (!NT_STATUS_IS_OK(status)) {
+			DBG_DEBUG("Could not read symlink for %s: %s\n",
+				  smb_fname_str_dbg(
+					  smb_fname_rel->fsp->fsp_name),
+				  nt_errstr(status));
 			goto fail;
 		}
-
-#if defined(WITH_SMB1SERVER)
-		/*
-		 * In SMB1 posix mode, if this is a symlink,
-		 * allow access to the name with a NULL smb_fname->fsp.
-		 */
-		if (ucf_flags & UCF_LCOMP_LNK_OK) {
-			SMB_ASSERT(smb_fname_rel->fsp == NULL);
-			SMB_ASSERT(streamname == NULL);
-
-			smb_fname = full_path_from_dirfsp_atname(
-				mem_ctx,
-				smb_dirname->fsp,
-				smb_fname_rel);
-			if (smb_fname == NULL) {
-				status = NT_STATUS_NO_MEMORY;
-				goto fail;
-			}
-			goto done;
-		}
-#endif
 	}
 
 	if (NT_STATUS_EQUAL(status, NT_STATUS_OBJECT_NAME_NOT_FOUND) &&
@@ -1403,12 +1030,15 @@ static NTSTATUS filename_convert_dirfsp_nosymlink(
 done:
 	*_dirfsp = smb_dirname->fsp;
 	*_smb_fname = smb_fname;
+	*_symlink_err = symlink_err;
 
 	smb_fname_fsp_unlink(smb_fname_rel);
-	TALLOC_FREE(smb_fname_rel);
+	*_smb_fname_rel = smb_fname_rel;
 	return NT_STATUS_OK;
 
 fail:
+	*_symlink_err = symlink_err;
+
 	/*
 	 * If open_stream_pathref_fsp() returns an error, smb_fname_rel->fsp
 	 * has been set to NULL, so we must free base_fsp separately
@@ -1426,44 +1056,78 @@ fail:
 	return status;
 }
 
-NTSTATUS filename_convert_dirfsp(
-	TALLOC_CTX *mem_ctx,
-	connection_struct *conn,
-	const char *name_in,
-	uint32_t ucf_flags,
-	NTTIME twrp,
-	struct files_struct **_dirfsp,
-	struct smb_filename **_smb_fname)
+NTSTATUS filename_convert_dirfsp_rel(TALLOC_CTX *mem_ctx,
+				     connection_struct *conn,
+				     struct files_struct *basedir,
+				     const char *name_in,
+				     uint32_t ucf_flags,
+				     NTTIME twrp,
+				     struct files_struct **_dirfsp,
+				     struct smb_filename **_smb_fname,
+				     struct smb_filename **_smb_fname_rel)
 {
-	char *substitute = NULL;
-	size_t unparsed = 0;
+	struct reparse_data_buffer *symlink_err = NULL;
+	struct symlink_reparse_struct *lnk = NULL;
 	NTSTATUS status;
 	char *target = NULL;
+	char *base_name = NULL;
 	char *safe_target = NULL;
 	size_t symlink_redirects = 0;
+	int ret;
 
 next:
 	if (symlink_redirects > 40) {
 		return NT_STATUS_OBJECT_PATH_NOT_FOUND;
 	}
 
-	status = filename_convert_dirfsp_nosymlink(
-		mem_ctx,
-		conn,
-		name_in,
-		ucf_flags,
-		twrp,
-		_dirfsp,
-		_smb_fname,
-		&substitute,
-		&unparsed);
+	status = filename_convert_dirfsp_nosymlink(mem_ctx,
+						   conn,
+						   basedir,
+						   name_in,
+						   ucf_flags,
+						   twrp,
+						   _dirfsp,
+						   _smb_fname,
+						   _smb_fname_rel,
+						   &symlink_err);
+
+	if (NT_STATUS_IS_OK(status) && S_ISLNK((*_smb_fname)->st.st_ex_mode)) {
+		/*
+		 * lcomp is a symlink
+		 */
+		if (ucf_flags & UCF_LCOMP_LNK_OK) {
+			TALLOC_FREE(symlink_err);
+			return NT_STATUS_OK;
+		}
+		close_file_free(NULL, _dirfsp, ERROR_CLOSE);
+		status = NT_STATUS_STOPPED_ON_SYMLINK;
+	}
 
 	if (!NT_STATUS_EQUAL(status, NT_STATUS_STOPPED_ON_SYMLINK)) {
 		return status;
 	}
+	TALLOC_FREE(*_smb_fname_rel);
+
+	lnk = &symlink_err->parsed.lnk;
+
+	/*
+	 * If we're on an MSDFS share, see if this is
+	 * an MSDFS link.
+	 */
+	if (lp_host_msdfs() && lp_msdfs_root(SNUM(conn)) &&
+	    strnequal(lnk->substitute_name, "msdfs:", 6))
+	{
+		TALLOC_FREE(*_smb_fname);
+		TALLOC_FREE(symlink_err);
+		return NT_STATUS_PATH_NOT_COVERED;
+	}
 
 	if (!lp_follow_symlinks(SNUM(conn))) {
-		return NT_STATUS_OBJECT_PATH_NOT_FOUND;
+		status = (lnk->unparsed_path_length == 0)
+				 ? NT_STATUS_OBJECT_NAME_NOT_FOUND
+				 : NT_STATUS_OBJECT_PATH_NOT_FOUND;
+		TALLOC_FREE(symlink_err);
+		return status;
 	}
 
 	/*
@@ -1478,28 +1142,64 @@ next:
 	 * resolve all symlinks locally.
 	 */
 
-	target = symlink_target_path(mem_ctx, name_in, substitute, unparsed);
-	if (target == NULL) {
-		return NT_STATUS_NO_MEMORY;
+	ret = symlink_target_path(mem_ctx,
+				  name_in,
+				  lnk->unparsed_path_length,
+				  lnk->substitute_name,
+				  lnk->substitute_name[0] != '/',
+				  '/',
+				  &target);
+	if (ret != 0) {
+		TALLOC_FREE(symlink_err);
+		return map_nt_error_from_unix(ret);
+	}
+
+	if (basedir != conn->cwd_fsp) {
+		base_name = basedir->fsp_name->base_name;
 	}
 
 	status = safe_symlink_target_path(mem_ctx,
 					  conn->connectpath,
-					  NULL,
+					  base_name,
 					  target,
-					  unparsed,
+					  lnk->unparsed_path_length,
 					  &safe_target);
-	TALLOC_FREE(target);
+	TALLOC_FREE(symlink_err);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
 	name_in = safe_target;
+	basedir = conn->cwd_fsp;
 
 	symlink_redirects += 1;
 
 	goto next;
 }
 
+NTSTATUS filename_convert_dirfsp(TALLOC_CTX *ctx,
+				 connection_struct *conn,
+				 const char *name_in,
+				 uint32_t ucf_flags,
+				 NTTIME twrp,
+				 struct files_struct **_dirfsp,
+				 struct smb_filename **_smb_name)
+{
+	struct smb_filename *smb_fname_rel = NULL;
+	NTSTATUS status;
+
+	status = filename_convert_dirfsp_rel(ctx,
+					     conn,
+					     conn->cwd_fsp,
+					     name_in,
+					     ucf_flags,
+					     twrp,
+					     _dirfsp,
+					     _smb_name,
+					     &smb_fname_rel);
+	TALLOC_FREE(smb_fname_rel);
+	return status;
+}
+
 char *full_path_from_dirfsp_at_basename(TALLOC_CTX *mem_ctx,
 					const struct files_struct *dirfsp,
 					const char *at_base_name)
