@@ -1361,7 +1361,7 @@ static bool test_finfo_after_write(struct torture_context *tctx, struct smbcli_s
 	sfinfo.basic_info.in.access_time = 0; \
 	unix_to_nt_time(&sfinfo.basic_info.in.write_time, (wrtime)); \
 	sfinfo.basic_info.in.change_time = 0; \
-	sfinfo.basic_info.in.attrib = finfo1.basic_info.out.attrib; \
+	sfinfo.basic_info.in.attrib = finfo.basic_info.out.attrib; \
 	_status = smb_raw_setfileinfo(tree, &sfinfo); \
 	if (!NT_STATUS_IS_OK(_status)) { \
 		torture_result(tctx, TORTURE_FAIL, __location__": setfileinfo failed: %s", \
@@ -1383,7 +1383,7 @@ static bool test_finfo_after_write(struct torture_context *tctx, struct smbcli_s
 	unix_to_nt_time(&sfinfo.basic_info.in.write_time, (wrtime)); \
 	sfinfo.basic_info.in.write_time += (ns); \
 	sfinfo.basic_info.in.change_time = 0; \
-	sfinfo.basic_info.in.attrib = finfo1.basic_info.out.attrib; \
+	sfinfo.basic_info.in.attrib = finfo.basic_info.out.attrib; \
 	_status = smb_raw_setfileinfo(tree, &sfinfo); \
 	if (!NT_STATUS_IS_OK(_status)) { \
 		torture_result(tctx, TORTURE_FAIL, __location__": setfileinfo failed: %s", \
@@ -1439,7 +1439,7 @@ static bool test_delayed_write_update3(struct torture_context *tctx,
 
 	/*
 	 * make sure the write time is updated 2 seconds later
-	 * calcuated from the first write
+	 * calculated from the first write
 	 * (but expect up to 5 seconds extra time for a busy server)
 	 */
 	start = timeval_current();
@@ -1601,7 +1601,7 @@ static bool test_delayed_write_update3a(struct torture_context *tctx,
 
 	/*
 	 * make sure the write time is updated 2 seconds later
-	 * calcuated from the first write
+	 * calculated from the first write
 	 * (but expect up to 5 seconds extra time for a busy server)
 	 */
 	start = timeval_current();
@@ -1812,7 +1812,7 @@ static bool test_delayed_write_update3b(struct torture_context *tctx,
 
 	/*
 	 * make sure the write time is updated 2 seconds later
-	 * calcuated from the first write
+	 * calculated from the first write
 	 * (but expect up to 5 seconds extra time for a busy server)
 	 */
 	start = timeval_current();
@@ -2187,7 +2187,7 @@ static bool test_delayed_write_update4(struct torture_context *tctx,
 
 	/*
 	 * make sure the write time is updated 2 seconds later
-	 * calcuated from the first write
+	 * calculated from the first write
 	 * (but expect up to 3 seconds extra time for a busy server)
 	 */
 	start = timeval_current();
@@ -2902,9 +2902,10 @@ static bool test_delayed_write_update7(struct torture_context *tctx, struct smbc
 	status = smb_raw_fileinfo(cli->tree, tctx, &finfo1);
 
 	torture_assert_ntstatus_ok(tctx, status, "fileinfo failed");
-	
-	/* Set the pending write time to a value with ns. */
-	SET_INFO_FILE_NS(finfo, time(NULL) + 86400, 103, cli->tree, fnum1);
+
+	/* Set the pending write time to a value with non zero msec. */
+	SET_INFO_FILE_NS(finfo1, time(NULL) + 86400, 103 * NTTIME_MSEC,
+			 cli->tree, fnum1);
 
 	/* Get the current pending write time by fnum. */
 	status = smb_raw_fileinfo(cli->tree, tctx, &finfo2);
